@@ -1,6 +1,6 @@
 /* Kernel crypto API AF_ALG Symmetric Cipher API
  *
- * Copyright (C) 2016, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2016 - 2017, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -51,7 +51,7 @@ int32_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 
 	/* require properly sized output data size */
 	if (outlen < ((inlen + bs - 1) / bs * bs))
-		kcapi_dolog(LOG_WARN,
+		kcapi_dolog(KCAPI_LOG_WARN,
 			    "Symmetric Encryption: Ciphertext buffer (%lu) is not plaintext buffer (%lu) rounded up to multiple of block size %u",
 			    (unsigned long) outlen, (unsigned long)inlen, bs);
 
@@ -85,15 +85,11 @@ int32_t kcapi_cipher_encrypt_aio(struct kcapi_handle *handle,
 
 	handle->cipher.iv = iv;
 
-	if (handle->aio.disable) {
-		ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
-					      access, ALG_OP_ENCRYPT);
+	ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
+				      access, ALG_OP_ENCRYPT);
+	if (ret != -EOPNOTSUPP)
+		return ret;
 
-		if (ret != -EOPNOTSUPP)
-			return ret;
-	}
-
-	/* The kernel's AIO interface is broken. */
 	return _kcapi_cipher_encrypt_aio_fallback(handle, iniov, outiov,
 						  iovlen, iv);
 }
@@ -106,12 +102,12 @@ int32_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
 {
 	/* require properly sized output data size */
 	if (inlen % handle->info.blocksize)
-		kcapi_dolog(LOG_WARN,
+		kcapi_dolog(KCAPI_LOG_WARN,
 			    "Symmetric Decryption: Ciphertext buffer is not multiple of block size %u",
 			    handle->info.blocksize);
 
 	if (outlen < inlen)
-		kcapi_dolog(LOG_WARN,
+		kcapi_dolog(KCAPI_LOG_WARN,
 			    "Symmetric Decryption: Plaintext buffer (%lu) is smaller as ciphertext buffer (%lu)",
 			    (unsigned long)outlen, (unsigned long)inlen);
 
@@ -145,15 +141,11 @@ int32_t kcapi_cipher_decrypt_aio(struct kcapi_handle *handle,
 
 	handle->cipher.iv = iv;
 
-	if (handle->aio.disable) {
-		ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
-					      access, ALG_OP_DECRYPT);
+	ret = _kcapi_cipher_crypt_aio(handle, iniov, outiov, iovlen,
+				      access, ALG_OP_DECRYPT);
+	if (ret != -EOPNOTSUPP)
+		return ret;
 
-		if (ret != -EOPNOTSUPP)
-			return ret;
-	}
-
-	/* The kernel's AIO interface is broken. */
 	return _kcapi_cipher_decrypt_aio_fallback(handle, iniov, outiov,
 						  iovlen, iv);
 }
@@ -194,7 +186,7 @@ int32_t kcapi_cipher_stream_op(struct kcapi_handle *handle,
 			       struct iovec *iov, uint32_t iovlen)
 {
 	if (!iov || !iovlen) {
-		kcapi_dolog(LOG_ERR,
+		kcapi_dolog(KCAPI_LOG_ERR,
 			    "Symmetric operation: No buffer for output data provided");
 		return -EINVAL;
 	}
@@ -212,3 +204,91 @@ uint32_t kcapi_cipher_blocksize(struct kcapi_handle *handle)
 {
 	return handle->info.blocksize;
 }
+
+static inline int32_t kcapi_cipher_conv_enc_common(const char *name,
+					const uint8_t *key, uint32_t keylen,
+					const uint8_t *in, uint32_t inlen,
+					const uint8_t *iv,
+					uint8_t *out, uint32_t outlen)
+{
+	struct kcapi_handle handle;
+	int32_t ret = _kcapi_allocated_handle_init(&handle, "skcipher", name,
+						   0);
+
+	if (ret)
+		return ret;
+
+	ret = kcapi_cipher_setkey(&handle, key, keylen);
+	if (ret)
+		goto out;
+
+	ret = kcapi_cipher_encrypt(&handle, in, inlen, iv, out, outlen, 0);
+
+out:
+	_kcapi_handle_destroy_nofree(&handle);
+	return ret;
+}
+
+static inline int32_t kcapi_cipher_conv_dec_common(const char *name,
+					const uint8_t *key, uint32_t keylen,
+					const uint8_t *in, uint32_t inlen,
+					const uint8_t *iv,
+					uint8_t *out, uint32_t outlen)
+{
+	struct kcapi_handle handle;
+	int32_t ret = _kcapi_allocated_handle_init(&handle, "skcipher", name,
+						   0);
+
+	if (ret)
+		return ret;
+
+	ret = kcapi_cipher_setkey(&handle, key, keylen);
+	if (ret)
+		goto out;
+
+	ret = kcapi_cipher_decrypt(&handle, in, inlen, iv, out, outlen, 0);
+
+out:
+	_kcapi_handle_destroy_nofree(&handle);
+	return ret;
+}
+
+DSO_PUBLIC
+int32_t kcapi_cipher_enc_aes_cbc(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *iv,
+				 uint8_t *out, uint32_t outlen)
+{
+	return kcapi_cipher_conv_enc_common("cbc(aes)", key, keylen, in, inlen,
+					    iv, out, outlen);
+}
+
+DSO_PUBLIC
+int32_t kcapi_cipher_enc_aes_ctr(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *ctr,
+				 uint8_t *out, uint32_t outlen)
+{
+	return kcapi_cipher_conv_enc_common("ctr(aes)", key, keylen, in, inlen,
+					    ctr, out, outlen);
+}
+
+DSO_PUBLIC
+int32_t kcapi_cipher_dec_aes_cbc(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *iv,
+				 uint8_t *out, uint32_t outlen)
+{
+	return kcapi_cipher_conv_dec_common("cbc(aes)", key, keylen, in, inlen,
+					    iv, out, outlen);
+}
+
+DSO_PUBLIC
+int32_t kcapi_cipher_dec_aes_ctr(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *ctr,
+				 uint8_t *out, uint32_t outlen)
+{
+	return kcapi_cipher_conv_dec_common("ctr(aes)", key, keylen, in, inlen,
+					    ctr, out, outlen);
+}
