@@ -17,8 +17,8 @@
  * DAMAGE.
  */
 
-#ifndef _KCAPI_H
-#define _KCAPI_H
+#ifndef KCAPI_H
+#define KCAPI_H
 
 #include <stdint.h>
 #include <sys/uio.h>
@@ -651,7 +651,7 @@ void kcapi_aead_setassoclen(struct kcapi_handle *handle, uint32_t assoclen);
  *	   a negative errno-style error code if an error occurred
  */
 int32_t kcapi_aead_encrypt(struct kcapi_handle *handle,
-			   uint8_t *in, uint32_t inlen,
+			   const uint8_t *in, uint32_t inlen,
 			   const uint8_t *iv,
 			   uint8_t *out, uint32_t outlen,
 			   int access);
@@ -798,7 +798,7 @@ void kcapi_aead_getdata_output(struct kcapi_handle *handle,
  *	   a negative errno-style error code if an error occurred
  */
 int32_t kcapi_aead_decrypt(struct kcapi_handle *handle,
-			   uint8_t *in, uint32_t inlen,
+			   const uint8_t *in, uint32_t inlen,
 			   const uint8_t *iv,
 			   uint8_t *out, uint32_t outlen, int access);
 
@@ -1183,6 +1183,8 @@ int kcapi_md_setkey(struct kcapi_handle *handle,
  * @buffer: [in] holding the data to add to the message digest
  * @len: [in] buffer length
  *
+ * The input buffer can be at most INT_MAX in size.
+ *
  * @return 0 upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -1220,6 +1222,8 @@ int32_t kcapi_md_final(struct kcapi_handle *handle,
  * The message digest handle must have been initialized, potentially by also
  * setting the key using the generic message digest API functions.
  *
+ * The input buffer can be at most INT_MAX in size.
+ *
  * @return size of message digest upon success;
  *	    -EIO - data cannot be obtained;
  * 	    -ENOMEM - buffer is too small for the complete message digest,
@@ -2561,4 +2565,4 @@ int32_t kcapi_hkdf(const char *hashname,
 }
 #endif
 
-#endif /* _KCAPI_H */
+#endif /* KCAPI_H */
